﻿using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesReturnCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesReturnCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        ISboProviderService m_sboProviderService;

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesReturnCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesReturnCreatedHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            string initialErrorKey = null;
            string numberRMA = null;
                
          
            Message msg = null;
         


            //Question to enter the RMA Numver
            goto Step_EnterRMANumber;

            

            Step_EnterRMANumber:

            #region ENTER RMA NUMBER

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_ENTER_RMA_NUMBER",
                    "ForceDataEntry", true,
                    "ForceUppercase", false,
                    "AllowToGoBack", true,
                    "Information", "MSG_ENTER_RMA_NUMBER"
                    ));
            msg = WaitForMessage();
         
            // reset error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringEntered"))
            {
                numberRMA = ExtractParameter<string>(msg.Parameters, "stringValue");
                if (string.IsNullOrEmpty(numberRMA))
                {
                    goto Step_EnterRMANumber;
                }
                //Save RMA number in field Customer Reference ORDN.NumAtCard
                goto Step_SaveRMANumber;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
               goto Step_EnterRMANumber;
            }


            #endregion

            Step_SaveRMANumber:

            #region SAVE RMA NUMBER


            m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                         delegate (PmxDbConnection conn, object[] parameters)
                         {
                             PmxSalesReturnProvider prov = new PmxSalesReturnProvider((SboConnection)conn);
                             PmxSalesReturn salesReturn= null;
                             salesReturn = prov.GetBO(DocumentRefs.Get()[0].DocEntry);

                             if (salesReturn != null)
                             {
                                 salesReturn.NumAtCard = numberRMA;

                                 prov.UpdateBO(salesReturn, false, true);

                             }
                            
                           
                             return null;

                         });

            #endregion
        }

        #endregion
    }
}
